==> sapdb-source-7.3.00.48.tgz: V73_00_48/SAPDB_ORG/sys/src/in/vin202.cpp <==
/*!================================================================
 module:        vin202.cpp

 -------------------------------------------------------------------

 responsible:   DanielD

 special area:  SQL

 description:   Segment

 -------------------------------------------------------------------

 copyright:     SAP AG 1999



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

 ===================================================================*/

#include "hin202.h"

/*----------------------------------------*/

static tsp1_packet *
in202_GetSegmentPacket (
    tsp1_segment * segment)
{
    int segmOffset;
    teo00_Byte * segmAddr;
    teo00_Byte * packetAddr;
    tsp1_packet * packet;

    segmOffset = segment->sp1s_segm_offset ();
    segmAddr = REINTERPRET_CAST (teo00_Byte*, segment);
    packetAddr = segmAddr - segmOffset - sizeof (tsp1_packet_header);
    packet = REINTERPRET_CAST (tsp1_packet*, packetAddr);
    return packet;
}

/*----------------------------------------*/

cin202_Segment::cin202_Segment (tsp1_segment * segmentArg)
{
    this->rawSegment = segmentArg;
}

/*----------------------------------------*/

cin202_Segment::~cin202_Segment ()
{
    this->rawSegment = NULL;
}

/*----------------------------------------*/

const char * segmKindStrings [] = {
    "nil",
    "cmd",
    "return"
};

const tin97_EnumDesc segmKindDesc = {
    sp1sk_return,
    "Invalid segment kind",
    segmKindStrings
};

const char * messtypeStrings [] = {
    "nil",
    "cmd_lower_bound",
    "dbs",
    "parse",
    "getparse",
    "syntax",
    "cfill1",
    "cfill2",
    "cfill3",
    "cfill4",
    "cfill5",
    "cmd_upper_bound",
    "no_cmd_lower_bound",
    "execute",
    "getexecute",
    "putval",
    "getval",
    "load",
    "unload",
    "ncfill1",
    "ncfill2",
    "ncfill3",
    "ncfill4",
    "ncfill5",
    "no_cmd_upper_bound",
    "hello",
    "util_lower_bound",
    "utility",
    "incopy",
    "ufill1",
    "outcopy",
    "diag_outcopy",
    "ufill3",
    "ufill4",
    "ufill5",
    "ufill6",
    "ufill7",
    "util_upper_bound",
    "specials_lower_bound",
    "switch",
    "switchlimit",
    "buflength",
    "minbuf",
    "maxbuf",
    "state_utility",
    "sfill2",
    "sfill3",
    "sfill4",
    "sfill5",
    "specials_upper_bound",
    "wait_for_event"
};

const tin97_EnumDesc messtypeDesc = {
    sp1m_wait_for_event,
    "Invalid mess type %d",
    messtypeStrings
};

const char * sqlModeStrings [] = {
    "nil",
    "session",
    "adabas",
    "ansi",
    "db2",
    "oracle"
};

const tin97_EnumDesc sqlModeDesc = {
    sp1sm_oracle,
    "Invalid SQL mode %d",
    sqlModeStrings
};

const char * producerStrings [] = {
    "nil",
    "user",
    "internal",
    "kernel",
    "installation"
};

const tin97_EnumDesc producerDesc = {
    sp1pr_installation,
    "invalid producer %d",
    producerStrings
};


/*----------------------------------------*/

void cin202_Segment::TraceOn (cin97_TraceStream * trace, int buflimit)
{
    tsp1_segment_header * header = &this->rawSegment->sp1s_segm_header ();
    char tmpBuf [tmpBufSize_cin97] [3];

    trace->printf ("   %s Segment %d at %d (%d of %d bytes)\n",
        in97EnumDump (&segmKindDesc, header->sp1s_segm_kind (), tmpBuf [0]),
        header->sp1s_own_index (), header->sp1s_segm_offset (),
        header->sp1s_segm_len (), this->Size ());
    switch (header->sp1s_segm_kind ()) {
        case sp1sk_cmd:
            trace->printf ("        messtype: %s  sqlmode: %s  producer: %s\n",
                in97EnumDump (&messtypeDesc, header->sp1c_mess_type (), tmpBuf [0]),
                in97EnumDump (&sqlModeDesc, header->sp1c_sqlmode (), tmpBuf [1]),
                in97EnumDump (&producerDesc, header->sp1c_producer (), tmpBuf [2]));
            trace->printf ("        Options: %s%s%s%s%s%s\n",
                header->sp1c_commit_immediately () ? "commit " : "",
                header->sp1c_ignore_costwarning () ? "ignore costwarning " : "",
                header->sp1c_prepare () ? "prepare " : "",
                header->sp1c_with_info () ? "with info " : "",
                header->sp1c_mass_cmd () ? "mass cmd " : "",
                header->sp1c_parsing_again () ? "parsing again " : "");
            break;
        case sp1sk_return:
            trace->printf (
		"        RC: %d  %.5s  (Pos %d) Function %d\n",
                (int)(header->sp1r_returncode ()),
		(char *)(header->sp1r_sqlstate ()),
                (int)(header->sp1r_errorpos ()),
		(int)(header->sp1r_function_code ()));
            break;
        default:
            trace->writeln ("unknown segement kind");
            break;
    }
    trace->printf ("        %d parts:\n", header->sp1s_no_of_parts ());

    cin202_ReplySegment tmpSegment (this->rawSegment);
    cin201_Part part = tmpSegment.GetFirstPart ();
    while (part.IsValid ()) {
        part.TraceOn (trace, buflimit);
        part = tmpSegment.GetNextPart (part);
    }
}

/*----------------------------------------*/

int cin202_Segment::Size () const
{
    int result;
    tsp1_packet_header * packetHeader;

    packetHeader = &in202_GetSegmentPacket (this->rawSegment)->sp1_header;
    result = packetHeader->sp1h_varpart_size
        - this->rawSegment->sp1s_segm_offset ();
    return result;
}

/*----------------------------------------*/

int cin202_Segment::Length () const
{
    return (this->rawSegment->sp1s_segm_len ());
}

/*----------------------------------------*/

bool cin202_Segment::IsValid () const
{
    return (this->rawSegment != NULL);
}

/*----------------------------------------*/

void cin202_Segment::Invalidate ()
{
    this->rawSegment = NULL;
}

/*----------------------------------------*/

tsp1_segment * cin202_Segment::GetRawSegment ()
{
    return this->rawSegment;
}

/*----------------------------------------*/

const tsp1_segment * cin202_Segment::GetRawSegment () const
{
    return this->rawSegment;
}

/*----------------------------------------*/

tsp1_segment_header * cin202_Segment::GetRawHeader ()
{
    return &this->rawSegment->sp1s_segm_header ();
}

/*----------------------------------------*/

const tsp1_segment_header * cin202_Segment::GetRawHeader () const
{
    return &this->rawSegment->sp1s_segm_header ();
}

/*----------------------------------------*/

cin202_RequestSegment::cin202_RequestSegment (tsp1_segment * segmParm)
    : cin202_Segment (segmParm)
{
}

/*----------------------------------------*/

/* PTS 1108446 sqlMode added      */
/* PTS 1112792 parsingAgain added */
void cin202_RequestSegment::Init (
    tsp1_cmd_mess_type_Param messType,
    const tsp1_sqlmode_Enum &sqlMode,
    const pasbool parsingAgain,
    int offset,
    int index)
{
    tsp1_segment_header * header = &this->rawSegment->sp1s_segm_header ();

    memset (header, 0, sizeof (tsp1_segment_header));
    header->sp1s_segm_len () = 0;
    header->sp1s_segm_offset () = offset;
    header->sp1s_own_index () = index;
    header->sp1s_segm_kind ().becomes (sp1sk_cmd);
    header->sp1c_mess_type ().becomes (messType);
//    header->sp1c_sqlmode ().becomes (sp1sm_session_sqlmode);
    header->sp1c_sqlmode ().becomes (sqlMode);      /* PTS 1108446 */
    header->sp1c_producer ().becomes (sp1pr_user_cmd);
    header->sp1c_parsing_again () = parsingAgain;
    // this will update packet as well
    this->ExtendLength (sizeof (tsp1_segment_header));
}

/*----------------------------------------*/

void cin202_RequestSegment::ExtendLength (int increment)
{
    this->rawSegment->sp1s_segm_len () += increment;
    in202_GetSegmentPacket (this->rawSegment)->sp1_header.sp1h_varpart_len += increment;
}

/*----------------------------------------*/

cin201_Part cin202_RequestSegment::AddPart (tsp1_part_kind_Param partKind)
{
    tsp1_part * partAddr;
    tsp1_segment_header * header = this->GetRawHeader ();
    teo00_Byte * rawPtr;

    // calculate part address
    rawPtr = REINTERPRET_CAST (teo00_Byte *, this->rawSegment)
        + this->rawSegment->sp1s_segm_len ();
    partAddr = REINTERPRET_CAST (tsp1_part *, rawPtr);
    // create Part object
    cin201_Part result (partAddr);
    int bytesRemaining = this->Size () - header->sp1s_segm_len ();
    result.Init (partKind, header->sp1s_segm_offset (), bytesRemaining);
    ++header->sp1s_no_of_parts ();
    this->ExtendLength (sizeof (tsp1_part_header));
    // return Part
    return result;
}

/*----------------------------------------*/

void cin202_RequestSegment::ClosePart (cin201_Part & part)
{
    int alignedLen = ALIGN_EO00 (part.Length (), packetAlignment_cin02);

    this->ExtendLength (alignedLen);
}

/*----------------------------------------*/

cin202_ReplySegment::cin202_ReplySegment (tsp1_segment * segmParm)
    : cin202_Segment (segmParm)
{
}

/*----------------------------------------*/

int cin202_ReplySegment::ErrorCode () const
{
    return this->rawSegment->sp1r_returncode ();
}

/*----------------------------------------*/

int cin202_ReplySegment::FunctionCode () const
{
    return this->rawSegment->sp1r_function_code ();
}

/*----------------------------------------*/

const cin201_Part cin202_ReplySegment::GetFirstPart () const
{
  tsp1_part * rawPart;
  if ( this->rawSegment->sp1s_segm_header().sp1s_no_of_parts() )
    rawPart = &this->rawSegment->sp1p_part ();
  else
    rawPart = NULL;
  return cin201_Part (rawPart);
}

/*----------------------------------------*/

const cin201_Part cin202_ReplySegment::GetNextPart (const cin201_Part & part) const
{
    // get address of next part
    int alignedLen = ALIGN_EO00 (part.Length (), packetAlignment_cin02);
    const tsp00_Byte * nextRawPart = part.GetReadData () + alignedLen;
    // check it's in range
    tsp00_Byte * endOfSegment = REINTERPRET_CAST (tsp00_Byte*, this->rawSegment)
        + this->rawSegment->sp1s_segm_len ();
    if (nextRawPart >= endOfSegment) {
        nextRawPart = NULL;
    }
    // convert pointer
    tsp1_part * rawPart = (tsp1_part *) nextRawPart;
    // return new Part
    return cin201_Part (rawPart);
}

/*----------------------------------------*/

const cin201_Part cin202_ReplySegment::FindPart (tsp1_part_kind_Param partKind) const
{
    cin201_Part part = this->GetFirstPart ();

    while (part.IsValid () && (part.GetRawHeader ()->sp1p_part_kind != partKind)) {
        part = this->GetNextPart (part);
    }
    // Exit reason 1: ! part.isValid () -> not found -> result is invalid
    // Exit reason 2: part.partKind = partKind -> result is ok
    return part;
}

/*----------------------------------------*/

int cin202_ReplySegment::FindParts (
    int count,
    const  tsp1_part_kind_Param partKinds [],
    cin201_Part parts []) const
{
    int i;
    int found = 0;
    tsp1_part_kind_Param partKind;

    for (i = 0; i < count; ++i) {
        parts [i].Invalidate ();
    }
    cin201_Part part = this->GetFirstPart ();
    while (part.IsValid ()) {
        partKind = part.GetRawHeader ()->sp1p_part_kind;
        for (i = 0; i < count; ++i) {
            if (partKind == partKinds [i]) {
                parts [i] = part;
                ++found;
                break;
            }
        }
        part = this->GetNextPart (part);
    }
    return found;
}

/*----------------------------------------*/

==> sapdb-source-7.4.03.32.tgz: V74_03_32/SAPDB_ORG/sys/src/in/vin202.cpp <==
/*!================================================================
 module:        vin202.cpp

 -------------------------------------------------------------------

 responsible:   DanielD

 special area:  SQL

 description:   Segment

 -------------------------------------------------------------------

 copyright:     SAP AG 1999



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

 ===================================================================*/

#include "hin202.h"

/*----------------------------------------*/

static tsp1_packet *
in202_GetSegmentPacket (
    tsp1_segment * segment)
{
    int segmOffset;
    teo00_Byte * segmAddr;
    teo00_Byte * packetAddr;
    tsp1_packet * packet;

    segmOffset = segment->sp1s_segm_offset ();
    segmAddr = REINTERPRET_CAST (teo00_Byte*, segment);
    packetAddr = segmAddr - segmOffset - sizeof (tsp1_packet_header);
    packet = REINTERPRET_CAST (tsp1_packet*, packetAddr);
    return packet;
}

/*----------------------------------------*/

cin202_Segment::cin202_Segment (tsp1_segment * segmentArg)
{
    this->rawSegment = segmentArg;
}

/*----------------------------------------*/

cin202_Segment::~cin202_Segment ()
{
    this->rawSegment = NULL;
}

/*----------------------------------------*/

const char * segmKindStrings [] = {
    "nil",
    "cmd",
    "return"
};

const tin97_EnumDesc segmKindDesc = {
    sp1sk_return,
    "Invalid segment kind",
    segmKindStrings
};

const char * messtypeStrings [] = {
    "nil",
    "cmd_lower_bound",
    "dbs",
    "parse",
    "getparse",
    "syntax",
    "cfill1",
    "cfill2",
    "cfill3",
    "cfill4",
    "cfill5",
    "cmd_upper_bound",
    "no_cmd_lower_bound",
    "execute",
    "getexecute",
    "putval",
    "getval",
    "load",
    "unload",
    "ncfill1",
    "ncfill2",
    "ncfill3",
    "ncfill4",
    "ncfill5",
    "no_cmd_upper_bound",
    "hello",
    "util_lower_bound",
    "utility",
    "incopy",
    "ufill1",
    "outcopy",
    "diag_outcopy",
    "ufill3",
    "ufill4",
    "ufill5",
    "ufill6",
    "ufill7",
    "util_upper_bound",
    "specials_lower_bound",
    "switch",
    "switchlimit",
    "buflength",
    "minbuf",
    "maxbuf",
    "state_utility",
    "sfill2",
    "sfill3",
    "sfill4",
    "sfill5",
    "specials_upper_bound",
    "wait_for_event"
};

const tin97_EnumDesc messtypeDesc = {
    sp1m_wait_for_event,
    "Invalid mess type %d",
    messtypeStrings
};

const char * sqlModeStrings [] = {
    "nil",
    "session",
    "adabas",
    "ansi",
    "db2",
    "oracle"
};

const tin97_EnumDesc sqlModeDesc = {
    sp1sm_oracle,
    "Invalid SQL mode %d",
    sqlModeStrings
};

const char * producerStrings [] = {
    "nil",
    "user",
    "internal",
    "kernel",
    "installation"
};

const tin97_EnumDesc producerDesc = {
    sp1pr_installation,
    "invalid producer %d",
    producerStrings
};


/*----------------------------------------*/

void cin202_Segment::TraceOn (cin97_TraceStream * trace, int buflimit)
{
    tsp1_segment_header * header = &this->rawSegment->sp1s_segm_header ();
    char tmpBuf [tmpBufSize_cin97] [3];

    trace->printf ("   %s Segment %d at %d (%d of %d bytes)\n",
        in97EnumDump (&segmKindDesc, header->sp1s_segm_kind (), tmpBuf [0]),
        header->sp1s_own_index (), header->sp1s_segm_offset (),
        header->sp1s_segm_len (), this->Size ());
    switch (sp1sk_cmd) {
        case sp1sk_cmd:
            trace->printf ("        messtype: %s  sqlmode: %s  producer: %s\n",
                in97EnumDump (&messtypeDesc, header->sp1c_mess_type (), tmpBuf [0]),
                in97EnumDump (&sqlModeDesc, header->sp1c_sqlmode (), tmpBuf [1]),
                in97EnumDump (&producerDesc, header->sp1c_producer (), tmpBuf [2]));
            trace->printf ("        Options: %s%s%s%s%s%s\n",
                header->sp1c_commit_immediately () ? "commit " : "",
                header->sp1c_ignore_costwarning () ? "ignore costwarning " : "",
                header->sp1c_prepare () ? "prepare " : "",
                header->sp1c_with_info () ? "with info " : "",
                header->sp1c_mass_cmd () ? "mass cmd " : "",
                header->sp1c_parsing_again () ? "parsing again " : "");
            break;
        case sp1sk_return:
            trace->printf ("        RC: %d  %.5s  (Pos %d) Function %d\n",
                header->sp1r_returncode (), (char *)(header->sp1r_sqlstate ()),
                header->sp1r_errorpos (), header->sp1r_function_code ());
            break;
        default:
            trace->writeln ("unknown segement kind");
            break;
    }
    trace->printf ("        %d parts:\n", header->sp1s_no_of_parts ());

    cin202_ReplySegment tmpSegment (this->rawSegment);
    cin201_Part part = tmpSegment.GetFirstPart ();
    while (part.IsValid ()) {
        part.TraceOn (trace, buflimit);
        part = tmpSegment.GetNextPart (part);
    }
}

/*----------------------------------------*/

int cin202_Segment::Size () const
{
    int result;
    tsp1_packet_header * packetHeader;

    packetHeader = &in202_GetSegmentPacket (this->rawSegment)->sp1_header;
    result = packetHeader->sp1h_varpart_size
        - this->rawSegment->sp1s_segm_offset ();
    return result;
}

/*----------------------------------------*/

int cin202_Segment::Length () const
{
    return (this->rawSegment->sp1s_segm_len ());
}

/*----------------------------------------*/

bool cin202_Segment::IsValid () const
{
    return (this->rawSegment != NULL);
}

/*----------------------------------------*/

void cin202_Segment::Invalidate ()
{
    this->rawSegment = NULL;
}

/*----------------------------------------*/

tsp1_segment * cin202_Segment::GetRawSegment ()
{
    return this->rawSegment;
}

/*----------------------------------------*/

const tsp1_segment * cin202_Segment::GetRawSegment () const
{
    return this->rawSegment;
}

/*----------------------------------------*/

tsp1_segment_header * cin202_Segment::GetRawHeader ()
{
    return &this->rawSegment->sp1s_segm_header ();
}

/*----------------------------------------*/

const tsp1_segment_header * cin202_Segment::GetRawHeader () const
{
    return &this->rawSegment->sp1s_segm_header ();
}

/*----------------------------------------*/

cin202_RequestSegment::cin202_RequestSegment (tsp1_segment * segmParm)
    : cin202_Segment (segmParm)
{
}

/*----------------------------------------*/

/* PTS 1108446 sqlMode added      */
/* PTS 1112690 parsingAgain added */
void cin202_RequestSegment::Init (
    tsp1_cmd_mess_type_Param messType,
    const tsp1_sqlmode_Enum &sqlMode,
    const pasbool parsingAgain,
    int offset,
    int index)
{
    tsp1_segment_header * header = &this->rawSegment->sp1s_segm_header ();

    memset (header, 0, sizeof (tsp1_segment_header));
    header->sp1s_segm_len () = 0;
    header->sp1s_segm_offset () = offset;
    header->sp1s_own_index () = index;
    header->sp1s_segm_kind ().becomes (sp1sk_cmd);
    header->sp1c_mess_type ().becomes (messType);
//    header->sp1c_sqlmode ().becomes (sp1sm_session_sqlmode);
    header->sp1c_sqlmode ().becomes (sqlMode);      /* PTS 1108446 */
    header->sp1c_producer ().becomes (sp1pr_user_cmd);
    header->sp1c_parsing_again () = parsingAgain;
    // this will update packet as well
    this->ExtendLength (sizeof (tsp1_segment_header));
}

/*----------------------------------------*/

void cin202_RequestSegment::ExtendLength (int increment)
{
    this->rawSegment->sp1s_segm_len () += increment;
    in202_GetSegmentPacket (this->rawSegment)->sp1_header.sp1h_varpart_len += increment;
}

/*----------------------------------------*/

cin201_Part cin202_RequestSegment::AddPart (tsp1_part_kind_Param partKind)
{
    tsp1_part * partAddr;
    tsp1_segment_header * header = this->GetRawHeader ();
    teo00_Byte * rawPtr;

    // calculate part address
    rawPtr = REINTERPRET_CAST (teo00_Byte *, this->rawSegment)
        + this->rawSegment->sp1s_segm_len ();
    partAddr = REINTERPRET_CAST (tsp1_part *, rawPtr);
    // create Part object
    cin201_Part result (partAddr);
    int bytesRemaining = this->Size () - header->sp1s_segm_len ();
    result.Init (partKind, header->sp1s_segm_offset (), bytesRemaining);
    ++header->sp1s_no_of_parts ();
    this->ExtendLength (sizeof (tsp1_part_header));
    // return Part
    return result;
}

/*----------------------------------------*/

void cin202_RequestSegment::ClosePart (cin201_Part & part)
{
    int alignedLen = ALIGN_EO00 (part.Length (), packetAlignment_cin02);

    this->ExtendLength (alignedLen);
}

/*----------------------------------------*/

cin202_ReplySegment::cin202_ReplySegment (tsp1_segment * segmParm)
    : cin202_Segment (segmParm)
{
}

/*----------------------------------------*/

int cin202_ReplySegment::ErrorCode () const
{
    return this->rawSegment->sp1r_returncode ();
}

/*----------------------------------------*/

int cin202_ReplySegment::FunctionCode () const
{
    return this->rawSegment->sp1r_function_code ();
}

/*----------------------------------------*/

const cin201_Part cin202_ReplySegment::GetFirstPart () const
{
  tsp1_part * rawPart;
  if ( this->rawSegment->sp1s_segm_header().sp1s_no_of_parts() )
    rawPart = &this->rawSegment->sp1p_part ();
  else
    rawPart = NULL;
  return cin201_Part (rawPart);
}

/*----------------------------------------*/

const cin201_Part cin202_ReplySegment::GetNextPart (const cin201_Part & part) const
{
    // get address of next part
    int alignedLen = ALIGN_EO00 (part.Length (), packetAlignment_cin02);
    const tsp00_Byte * nextRawPart = part.GetReadData () + alignedLen;
    // check it's in range
    tsp00_Byte * endOfSegment = REINTERPRET_CAST (tsp00_Byte*, this->rawSegment)
        + this->rawSegment->sp1s_segm_len ();
    if (nextRawPart >= endOfSegment) {
        nextRawPart = NULL;
    }
    // convert pointer
    tsp1_part * rawPart = (tsp1_part *) nextRawPart;
    // return new Part
    return cin201_Part (rawPart);
}

/*----------------------------------------*/

const cin201_Part cin202_ReplySegment::FindPart (tsp1_part_kind_Param partKind) const
{
    cin201_Part part = this->GetFirstPart ();

    while (part.IsValid () && (part.GetRawHeader ()->sp1p_part_kind != partKind)) {
        part = this->GetNextPart (part);
    }
    // Exit reason 1: ! part.isValid () -> not found -> result is invalid
    // Exit reason 2: part.partKind = partKind -> result is ok
    return part;
}

/*----------------------------------------*/

int cin202_ReplySegment::FindParts (
    int count,
    const  tsp1_part_kind_Param partKinds [],
    cin201_Part parts []) const
{
    int i;
    int found = 0;
    tsp1_part_kind_Param partKind;

    for (i = 0; i < count; ++i) {
        parts [i].Invalidate ();
    }
    cin201_Part part = this->GetFirstPart ();
    while (part.IsValid ()) {
        partKind = part.GetRawHeader ()->sp1p_part_kind;
        for (i = 0; i < count; ++i) {
            if (partKind == partKinds [i]) {
                parts [i] = part;
                ++found;
                break;
            }
        }
        part = this->GetNextPart (part);
    }
    return found;
}

/*----------------------------------------*/
